"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientEventbridge = require("@aws-sdk/client-eventbridge");
var _bluebird = _interopRequireDefault(require("bluebird"));
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _utilRetry = require("@smithy/util-retry");
var _lodash = require("lodash");
var _retry = require("../utils/retry");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

class Connector {
  constructor({
    debug,
    pipelineId,
    timeout = Number(process.env.BUS_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig,
    additionalClientOpts = {},
    ...opt
  }) {
    this.debug = msg => debug('%j', msg);
    this.client = Connector.getClient(pipelineId, debug, timeout, additionalClientOpts);
    this.retryConfig = retryConfig;
    this.opt = opt;
  }
  static clients = {};
  static getClient(pipelineId, debug, timeout, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      this.clients[pipelineId] = new _clientEventbridge.EventBridgeClient({
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        retryStrategy: new _utilRetry.ConfiguredRetryStrategy(11, _retry.defaultBackoffDelay),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
    }
    return this.clients[pipelineId];
  }
  putEvents(params, ctx) {
    return this._putEvents(params, [], ctx);
  }
  _putEvents(params, attempts, ctx) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this._sendCommand(new _clientEventbridge.PutEventsCommand(params), ctx).then(resp => {
      if (resp.FailedEntryCount > 0) {
        return this._putEvents(unprocessed(params, resp), [...attempts, resp], ctx);
      } else {
        return accumlate(attempts, resp);
      }
    }));
  }
  _sendCommand(command, ctx) {
    this.opt.metrics?.capture(this.client, command, 'eventbridge', this.opt, ctx);
    return _bluebird.default.resolve(this.client.send(command)).tap(this.debug).tapCatch(this.debug);
  }
}
var _default = exports.default = Connector;
const unprocessed = (params, resp) => ({
  ...params,
  Entries: params.Entries.filter((e, i) => resp.Entries[i].ErrorCode)
});
const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({
  ...a,
  Entries: [...c.Entries.filter(e => !e.ErrorCode), ...a.Entries.filter(e => !e.ErrorCode)],
  attempts: [...attempts, resp]
}), resp);