"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientSecretsManager = require("@aws-sdk/client-secrets-manager");
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _bluebird = _interopRequireDefault(require("bluebird"));
var _lodash = require("lodash");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

const defaultDecodeFn = encoded => JSON.parse(Buffer.from(encoded, 'base64').toString());

/**
 * All secrets are combined to reduce the number of calls
 * Secrets are transferred from a ci/cd pipeline's secured variables
 * to SecretsManager using the serverless-secrets-mgr-plugin
 */
class Connector {
  constructor({
    debug,
    pipelineId,
    secretId,
    decodeFn = defaultDecodeFn,
    timeout = Number(process.env.SECRETSMGR_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    additionalClientOpts = {}
  }) {
    this.debug = /* istanbul ignore next */msg => debug('%j', msg);
    this.secretId = secretId;
    this.client = Connector.getClient(pipelineId, debug, timeout, additionalClientOpts);
    this.decodeFn = decodeFn;
  }
  static clients = {};
  static getClient(pipelineId, debug, timeout, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      this.clients[pipelineId] = new _clientSecretsManager.SecretsManagerClient({
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
    }
    return this.clients[pipelineId];
  }
  async get() {
    if (!this.secrets) {
      const params = {
        SecretId: this.secretId
      };
      this.secrets = await _bluebird.default.resolve(this.client.send(new _clientSecretsManager.GetSecretValueCommand(params))).tapCatch(this.debug).then(data => this.decodeFn(data.SecretString));
    }
    return this.secrets;
  }
}
var _default = exports.default = Connector;