"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toCursorUpdateRequest = exports.job = exports.flushCursor = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _utils = require("../utils");
var _dynamodb = require("../queries/dynamodb");
var _dynamodb2 = require("../sinks/dynamodb");
var _filters = require("../filters");
const job = rule => s => s // eslint-disable-line import/prefer-default-export
.filter(onEventType(rule)).tap(_utils.printStartPipeline)

// job level filter
.filter(onContent({
  ...rule,
  filters: rule.jobFilters
}))

// scan for records of interest
.map(toScanRequest(rule)).through((0, _dynamodb.scanSplitDynamoDB)(rule))

// or query for records of interest
.map(toQuerySplitRequest(rule)).through((0, _dynamodb.querySplitDynamoDB)(rule))

// current uow level filter
.filter(onContent(rule))

// query related data for the current uow
.map(toQueryRelatedRequest(rule)).through((0, _dynamodb.queryAllDynamoDB)(rule)).through((0, _utils.splitObject)({
  splitTargetField: rule.queryResponseField || 'queryResponse',
  ...rule
}))

// get related data for the current uow
.map(toGetRequest(rule)).through((0, _dynamodb.batchGetDynamoDB)(rule))

// write back to the db
.map(toUpdateRequest(rule)).through((0, _dynamodb2.updateDynamoDB)(rule))

// or publish an event
.map(toEvent(rule)).parallel(rule.parallel || Number(process.env.PARALLEL) || 4).through((0, _utils.encryptEvent)({
  ...rule,
  sourceField: 'emit',
  targetField: 'emit'
})).through(rule.publish({
  ...rule,
  parallel: 10,
  eventField: 'emit' // so we don't overwrite the incoming event in the uow
})).through(flushCursor(rule)).tap(_utils.printEndPipeline);
exports.job = job;
const onEventType = rule => (0, _utils.faulty)(uow => (0, _filters.filterOnEventType)(rule, uow));
const toScanRequest = rule => (0, _utils.faulty)(uow => ({
  ...uow,
  scanRequest: rule.toScanRequest ? /* istanbul ignore next */rule.toScanRequest(uow, rule) : undefined
}));
const toQuerySplitRequest = rule => (0, _utils.faulty)(uow => ({
  ...uow,
  querySplitRequest: rule.toQuerySplitRequest ? rule.toQuerySplitRequest(uow, rule) : /* istanbul ignore next */undefined
}));
const onContent = rule => (0, _utils.faulty)(uow => (0, _filters.filterOnContent)(rule, uow));
const toQueryRelatedRequest = rule => (0, _utils.faulty)(uow => ({
  ...uow,
  queryRequest: /* istanbul ignore next */
  rule.toQueryRelatedRequest || rule.toQueryRequest ? (rule.toQueryRelatedRequest || rule.toQueryRequest)(uow, rule) : undefined
}));
const toGetRequest = rule => (0, _utils.faulty)(uow => ({
  ...uow,
  batchGetRequest: rule.toGetRequest ? /* istanbul ignore next */rule.toGetRequest(uow, rule) : undefined
}));
const toUpdateRequest = rule => (0, _utils.faulty)(uow => ({
  ...uow,
  updateRequest: rule.toUpdateRequest ? /* istanbul ignore next */rule.toUpdateRequest(uow, rule) : undefined
}));
const toEvent = rule => (0, _utils.faultyAsyncStream)(async uow => !rule.toEvent ? /* istanbul ignore next */uow : {
  ...uow,
  emit: {
    ...(await (0, _utils.faultify)(rule.toEvent)(uow, rule))
  }
});
const toCursorUpdateRequest = rule => (0, _utils.faulty)(uow => ({
  ...uow,
  cursorUpdateRequest: rule.toCursorUpdateRequest ? rule.toCursorUpdateRequest(uow, rule) : /* istanbul ignore next */undefined
}));
exports.toCursorUpdateRequest = toCursorUpdateRequest;
const flushCursor = rule => s => {
  let lastUow;
  const cursorStream = () => (0, _highland.default)([lastUow]).map(toCursorUpdateRequest(rule)).through((0, _dynamodb2.updateDynamoDB)({
    ...rule,
    updateRequestField: 'cursorUpdateRequest',
    updateResponseField: 'cursorUpdateResponse'
  }));

  /* istanbul ignore else */
  if (rule.toCursorUpdateRequest) {
    return s.consume((err, x, push, next) => {
      /* istanbul ignore if */
      if (err) {
        push(err);
        next();
      } else if (x === _highland.default.nil) {
        if (lastUow) {
          next(cursorStream());
        } else {
          push(null, x);
        }
      } else {
        lastUow = x;
        push(null, x);
        next();
      }
    });
  } else {
    return s;
  }
};
exports.flushCursor = flushCursor;