"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toKinesisRecords = exports.fromKinesis = exports.UNKNOWN_KINESIS_EVENT_TYPE = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _utils = require("../utils");
var _filters = require("../filters");
var _queries = require("../queries");
const fromKinesis = event => (0, _highland.default)(event.Records).map(record => (
// create a unit-of-work for each event
// so we can correlate related work for error handling
{
  record,
  event: Buffer.from(record.kinesis.data, 'base64').toString('utf8')
})).map((0, _utils.faulty)(uow => ({
  ...uow,
  event: {
    id: uow.record.eventID,
    ...JSON.parse(uow.event, _utils.decompress)
  }
}))).tap(uow => (0, _utils.options)().metrics?.adornKinesisMetrics(uow, event)).filter(_filters.outSkip).through((0, _queries.redeemClaimcheck)());

// test helper
exports.fromKinesis = fromKinesis;
const toKinesisRecords = (events, approximateArrivalTimestamp) => ({
  Records: events.map((e, i) => ({
    eventSource: 'aws:kinesis',
    // eventVersion: '1.0',
    eventID: `shardId-000000000000:${i}`,
    // eventName: 'aws:kinesis:record',
    // invokeIdentityArn: 'arn:aws:iam::123456789012:role/lambda-role',
    awsRegion: process.env.AWS_REGION || /* istanbul ignore next */'us-west-2',
    // eventSourceARN: 'arn:aws:kinesis:us-west-2:123456789012:stream/lambda-stream',
    kinesis: {
      // kinesisSchemaVersion: '1.0',
      // partitionKey: e.partitionKey,
      sequenceNumber: `${i}`,
      data: Buffer.from(JSON.stringify(e, (0, _utils.compress)())).toString('base64'),
      approximateArrivalTimestamp // format: 1545084650.987
    }
  }))
});
exports.toKinesisRecords = toKinesisRecords;
const UNKNOWN_KINESIS_EVENT_TYPE = exports.UNKNOWN_KINESIS_EVENT_TYPE = toKinesisRecords([{
  type: 'unknown-type'
}]);