"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toPkQueryRequest = exports.toIndexQueryRequest = exports.toGetRequest = exports.scanSplitDynamoDB = exports.querySplitDynamoDB = exports.queryAllDynamoDB = exports.batchGetDynamoDB = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _memoryCache = _interopRequireDefault(require("memory-cache"));
var _dynamodb = _interopRequireDefault(require("../connectors/dynamodb"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
const batchGetDynamoDB = ({
  id: pipelineId,
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  batchGetRequestField = 'batchGetRequest',
  batchGetResponseField = 'batchGetResponse',
  parallel = Number(process.env.GET_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
  decrypt = async data => data,
  step = 'get',
  ...opt
} = {}) => {
  const connector = new _dynamodb.default({
    pipelineId,
    debug,
    tableName,
    timeout,
    ...opt
  });
  const invoke = uow => {
    if (!uow[batchGetRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const req = JSON.stringify(uow[batchGetRequestField]);
    const cached = _memoryCache.default.get(req);
    const p = () => (cached ? /* istanbul ignore next */Promise.resolve(cached) : connector.batchGet(uow[batchGetRequestField], uow).then(async batchGetResponse => ({
      ...batchGetResponse,
      Responses: await Object.keys(batchGetResponse.Responses).reduce(async (a, c) => {
        a = await a;
        return {
          ...a,
          [c]: await Promise.all(batchGetResponse.Responses[c]?.map(decrypt))
        };
      }, {})
    })).then(batchGetResponse => {
      _memoryCache.default.put(req, batchGetResponse);
      return batchGetResponse;
    })).then(batchGetResponse => ({
      ...uow,
      [batchGetResponseField]: batchGetResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(invoke).parallel(parallel);
};
exports.batchGetDynamoDB = batchGetDynamoDB;
const queryAllDynamoDB = ( /* istanbul ignore next */{
  id: pipelineId,
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  queryRequestField = 'queryRequest',
  queryResponseField = 'queryResponse',
  parallel = Number(process.env.QUERY_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
  decrypt = async data => data,
  step = 'query',
  ...opt
} = {}) => {
  const connector = new _dynamodb.default({
    pipelineId,
    debug,
    tableName,
    timeout,
    ...opt
  });
  const invoke = uow => {
    if (!uow[queryRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const req = JSON.stringify(uow[queryRequestField]);
    const cached = _memoryCache.default.get(req);
    const p = () => (cached ? Promise.resolve(cached) : connector.query(uow[queryRequestField], uow).then(queryResponse => Promise.all(queryResponse.map(decrypt))).then(queryResponse => {
      _memoryCache.default.put(req, queryResponse);
      return queryResponse;
    })).then(queryResponse => ({
      ...uow,
      [queryResponseField]: queryResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(invoke).parallel(parallel);
};
exports.queryAllDynamoDB = queryAllDynamoDB;
const toPkQueryRequest = (uow, rule) => ({
  KeyConditionExpression: '#pk = :pk',
  ExpressionAttributeNames: {
    '#pk': rule.pkFn || 'pk'
  },
  ExpressionAttributeValues: {
    ':pk': uow.event.partitionKey
  },
  ConsistentRead: true
});
exports.toPkQueryRequest = toPkQueryRequest;
const toIndexQueryRequest = (uow, rule) => ({
  IndexName: rule.indexNm,
  KeyConditionExpression: '#pk = :pk',
  ExpressionAttributeNames: {
    '#pk': rule.indexFn
  },
  ExpressionAttributeValues: {
    ':pk': uow.event.partitionKey
  },
  ConsistentRead: false
});
exports.toIndexQueryRequest = toIndexQueryRequest;
const toGetRequest = (uow, rule) => {
  const data = uow.event.raw.new || /* istanbul ignore next */uow.event.raw.old;
  const Keys = rule.fks.reduce((a, fk) => {
    const value = data[fk];
    /* istanbul ignore else */
    if (value) {
      const [discriminator, pk] = value.split('|');
      return [...a, {
        pk,
        sk: discriminator
      }];
    } else {
      return a;
    }
  }, []);
  return {
    RequestItems: {
      [rule.tableName]: {
        Keys
      }
    }
  };
};
exports.toGetRequest = toGetRequest;
const scanSplitDynamoDB = ({
  id: pipelineId,
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME || process.env.TABLE_NAME,
  scanRequestField = 'scanRequest',
  scanResponseField = 'scanResponse',
  parallel = Number(process.env.SCAN_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
  decrypt = async data => data,
  step = 'scan',
  ...opt
} = {}) => {
  const connector = new _dynamodb.default({
    pipelineId,
    debug,
    tableName,
    timeout,
    ...opt
  });
  const scan = uow => {
    if (!uow[scanRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    let cursor = uow[scanRequestField].ExclusiveStartKey;
    let itemsCount = 0;
    return (0, _highland.default)((push, next) => {
      const params = {
        ...uow[scanRequestField],
        ExclusiveStartKey: cursor
      };
      const p = () => connector.scan(params, uow);
      (uow.metrics?.w(p, step) || p()).then(async ({
        LastEvaluatedKey,
        Items,
        ...rest
      }) => ({
        LastEvaluatedKey,
        Items: await Promise.all(Items.map(decrypt)),
        ...rest
      })).then(data => {
        const {
          LastEvaluatedKey,
          Items,
          ...rest
        } = data;
        itemsCount += Items.length;
        if (LastEvaluatedKey && (!params.Limit || params.Limit && itemsCount < params.Limit)) {
          cursor = LastEvaluatedKey;
        } else {
          cursor = undefined;
        }
        Items.forEach(Item => {
          push(null, {
            ...uow,
            [scanRequestField]: params,
            [scanResponseField]: {
              ...rest,
              LastEvaluatedKey,
              Item
            }
          });
        });
      }).catch( /* istanbul ignore next */err => {
        err.uow = uow;
        push(err, null);
      }).finally(() => {
        if (cursor) {
          next();
        } else {
          push(null, _highland.default.nil);
        }
      });
    });
  };
  return s => s.map(scan).parallel(parallel);
};
exports.scanSplitDynamoDB = scanSplitDynamoDB;
const querySplitDynamoDB = ({
  id: pipelineId,
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME || process.env.TABLE_NAME,
  querySplitRequestField = 'querySplitRequest',
  querySplitResponseField = 'querySplitResponse',
  parallel = Number(process.env.SCAN_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
  decrypt = async data => data,
  step = 'query',
  ...opt
} = {}) => {
  const connector = new _dynamodb.default({
    pipelineId,
    debug,
    tableName,
    timeout,
    ...opt
  });
  const invoke = uow => {
    if (!uow[querySplitRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    let cursor = uow[querySplitRequestField].ExclusiveStartKey;
    let itemsCount = 0;
    return (0, _highland.default)((push, next) => {
      const params = {
        ...uow[querySplitRequestField],
        ExclusiveStartKey: cursor
      };
      const p = () => connector.queryPage(params, uow);
      (uow.metrics?.w(p, step) || p()).then(async ({
        LastEvaluatedKey,
        Items,
        ...rest
      }) => ({
        LastEvaluatedKey,
        Items: await Promise.all(Items.map(decrypt)),
        ...rest
      })).then(data => {
        const {
          LastEvaluatedKey,
          Items,
          ...rest
        } = data;
        itemsCount += Items.length;
        if (LastEvaluatedKey && (!params.Limit || params.Limit && itemsCount < params.Limit)) {
          cursor = LastEvaluatedKey;
        } else {
          cursor = undefined;
        }
        Items.forEach(Item => {
          push(null, {
            ...uow,
            [querySplitRequestField]: params,
            [querySplitResponseField]: {
              ...rest,
              LastEvaluatedKey,
              Item
            }
          });
        });
      }).catch( /* istanbul ignore next */err => {
        err.uow = uow;
        push(err, null);
      }).finally(() => {
        if (cursor) {
          next();
        } else {
          push(null, _highland.default.nil);
        }
      });
    });
  };
  return s => s.map(invoke).parallel(parallel);
};
exports.querySplitDynamoDB = querySplitDynamoDB;