"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toGetObjectRequest2 = exports.toGetObjectRequest = exports.splitS3Object = exports.pageObjectsFromS3 = exports.listObjectsFromS3 = exports.getObjectFromS3AsStream = exports.getObjectFromS3 = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _s = _interopRequireDefault(require("../connectors/s3"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
const toGetObjectRequest = uow => ({
  ...uow,
  getRequest: {
    Bucket: uow.record.s3.bucket.name,
    Key: uow.record.s3.object.key
  }
});
exports.toGetObjectRequest = toGetObjectRequest;
const toGetObjectRequest2 = uow => ({
  ...uow,
  getRequest: {
    Bucket: uow.record.s3.s3.bucket.name,
    Key: uow.record.s3.s3.object.key
  }
});
exports.toGetObjectRequest2 = toGetObjectRequest2;
const getObjectFromS3 = ({
  id: pipelineId,
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  getRequestField = 'getRequest',
  getResponseField = 'getResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'get',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const getObject = uow => {
    if (!uow[getRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.getObject(uow[getRequestField], uow).then(getResponse => ({
      ...uow,
      [getResponseField]: getResponse
    })) // TODO decompress
    .catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(getObject).parallel(parallel);
};
exports.getObjectFromS3 = getObjectFromS3;
const getObjectFromS3AsStream = ({
  id: pipelineId,
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  getRequestField = 'getRequest',
  getResponseField = 'getResponse',
  delimiter = '\n',
  splitFilter = () => true,
  step = 'get',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const getObject = uow => {
    if (!uow[getRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.getObjectStream(uow[getRequestField], uow);
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()) // wrap promise in a stream
    .flatMap(readable => (0, _highland.default)(readable)) // wrap stream in a stream
    .splitBy(delimiter).filter(splitFilter).map(getResponse => ({
      ...uow,
      [getResponseField]: getResponse
    }));
  };
  return s => s.flatMap(getObject);
};
exports.getObjectFromS3AsStream = getObjectFromS3AsStream;
const splitS3Object = ({
  delimiter = '\n',
  getResponseField = 'getResponse'
} = {}) => uow => {
  const {
    Body,
    ...rest
  } = uow[getResponseField];
  return (0, _highland.default)(Buffer.from(Body).toString().split(delimiter).filter(line => line.length !== 0).map(line => ({
    ...uow,
    [getResponseField]: {
      ...rest,
      line
    }
  })));
};
exports.splitS3Object = splitS3Object;
const listObjectsFromS3 = ({
  id: pipelineId,
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  listRequestField = 'listRequest',
  listResponseField = 'listResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'list',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const listObjects = uow => {
    /* istanbul ignore if */
    if (!uow[listRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.listObjects(uow[listRequestField], uow).then(listResponse => ({
      ...uow,
      [listResponseField]: listResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(listObjects).parallel(parallel);
};
exports.listObjectsFromS3 = listObjectsFromS3;
const pageObjectsFromS3 = ({
  id: pipelineId,
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  listRequestField = 'listRequest',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'list',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const listObjects = uow => {
    let {
      ContinuationToken
    } = uow[listRequestField];
    return (0, _highland.default)((push, next) => {
      const params = {
        ...uow[listRequestField],
        ContinuationToken
      };
      const p = () => connector.listObjects(params, uow);
      (uow.metrics?.w(p, step) || p()).then(data => {
        const {
          Contents,
          ...rest
        } = data;
        debug('listObjects: %j', rest);
        if (rest.IsTruncated) {
          ContinuationToken = rest.NextContinuationToken;
        } else {
          ContinuationToken = undefined;
        }
        Contents.forEach(obj => {
          push(null, {
            ...uow,
            [listRequestField]: params,
            listResponse: {
              ...rest,
              Content: obj
            }
          });
        });
      }).catch( /* istanbul ignore next */err => {
        err.uow = uow;
        push(err, null);
      }).finally(() => {
        if (ContinuationToken) {
          next();
        } else {
          push(null, _highland.default.nil);
        }
      });
    });
  };
  return s => s.map(listObjects).parallel(parallel);
};
exports.pageObjectsFromS3 = pageObjectsFromS3;