"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toPutClaimcheckRequest = exports.toClaimcheckEvent = exports.storeClaimcheck = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _s = require("./s3");
var _time = require("../utils/time");
// claim-check pattern support
// https://www.enterpriseintegrationpatterns.com/patterns/messaging/StoreInLibrary.html

const formatKey = event => {
  const d = new Date((0, _time.now)());
  // region/claimchecks/YYYY/MM/DD/HH/id
  return `${process.env.AWS_REGION}/claimchecks/${d.getUTCFullYear()}/${d.getUTCMonth()}/${d.getUTCDate()}/${d.getUTCHours()}/${event.id}`;
};
const toClaimcheckEvent = (event, bucket) => ({
  id: event.id,
  type: event.type,
  partitionKey: event.partitionKey,
  timestamp: event.timestamp,
  tags: event.tags,
  s3: {
    bucket,
    key: formatKey(event)
  }
});
exports.toClaimcheckEvent = toClaimcheckEvent;
const toPutClaimcheckRequest = (event, Bucket) => ({
  Bucket,
  Key: formatKey(event),
  Body: JSON.stringify(event)
});

// designed to run right after batchWithSize
exports.toPutClaimcheckRequest = toPutClaimcheckRequest;
const storeClaimcheck = ({
  id: pipelineId,
  // usually the s3 event lake bucket
  claimCheckBucketName = process.env.CLAIMCHECK_BUCKET_NAME,
  putClaimcheckRequest = 'putClaimcheckRequest',
  putClaimcheckResponse = 'putClaimcheckResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  debug,
  step,
  ...opt
}) => {
  // if we don't have a bucket we can't claimcheck
  if (!claimCheckBucketName) return s => s;
  return s => s.flatMap(batch => (0, _highland.default)(batch).through((0, _s.putObjectToS3)({
    debug,
    id: pipelineId,
    bucketName: claimCheckBucketName,
    putRequestField: putClaimcheckRequest,
    putResponseField: putClaimcheckResponse,
    parallel,
    step,
    ...opt
  }))
  // reassemble the batch
  .collect());
  // minus any faults during putObject
};
exports.storeClaimcheck = storeClaimcheck;