"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toRecord = exports.publishToKinesis = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _kinesis = _interopRequireDefault(require("../connectors/kinesis"));
var _batch = require("../utils/batch");
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _tags = require("../utils/tags");
var _compression = require("../utils/compression");
var _ratelimit = require("../utils/ratelimit");
const publishToKinesis = ({
  id: pipelineId,
  debug = (0, _print.debug)('kinesis'),
  streamName = process.env.STREAM_NAME,
  eventField = 'event',
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 25,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true,
  step = 'publish',
  ...opt
} = {}) => {
  const connector = new _kinesis.default({
    pipelineId,
    debug,
    streamName,
    ...opt
  });
  const toInputParams = batchUow => ({
    ...batchUow,
    inputParams: {
      Records: batchUow.batch.filter(uow => uow[eventField]).map(uow => toRecord(uow[eventField], opt))
    }
  });
  const putRecords = batchUow => {
    if (!batchUow.inputParams.Records.length) {
      return (0, _highland.default)(Promise.resolve(batchUow));
    }
    const p = () => connector.putRecords(batchUow.inputParams, batchUow).then(publishResponse => ({
      ...batchUow,
      publishResponse
    })).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors));
    return (0, _highland.default)(batchUow.batch[0].metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map((0, _tags.adornStandardTags)(eventField)).through((0, _ratelimit.ratelimit)(opt)).batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(putRecords).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};
exports.publishToKinesis = publishToKinesis;
const toRecord = (e, opt) => ({
  Data: Buffer.from(JSON.stringify(e, (0, _compression.compress)(opt))),
  PartitionKey: e.partitionKey
});
exports.toRecord = toRecord;