"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateEventSourceMapping = exports.invokeLambda = exports.deleteEventSourceMapping = exports.createEventSourceMapping = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _lambda = _interopRequireDefault(require("../connectors/lambda"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _ratelimit = require("../utils/ratelimit");
const invokeLambda = ({
  // eslint-disable-line import/prefer-default-export
  id: pipelineId,
  debug = (0, _print.debug)('lambda'),
  invokeField = 'invokeRequest',
  parallel = Number(process.env.LAMBDA_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'invoke',
  ...opt
} = {}) => {
  const connector = new _lambda.default({
    pipelineId,
    debug,
    ...opt
  });
  const invoke = uow => {
    const p = () => connector.invoke(uow[invokeField], uow).then(invokeResponse => ({
      ...uow,
      invokeResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(invoke).parallel(parallel);
};
exports.invokeLambda = invokeLambda;
const createEventSourceMapping = ({
  id: pipelineId,
  debug = (0, _print.debug)('lambda'),
  step = 'save',
  ...opt
} = {}) => {
  const connector = new _lambda.default({
    pipelineId,
    debug,
    ...opt
  });
  const invoke = uow => {
    if (!uow.createRequest) return (0, _highland.default)(Promise.resolve(uow));
    const {
      region,
      ...params
    } = uow.createRequest;
    connector.client.config.region = region || process.env.AWS_REGION;
    const p = () => connector.createEventSourceMapping(params, uow).then(createResponse => ({
      ...uow,
      createResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.flatMap(invoke);
};
exports.createEventSourceMapping = createEventSourceMapping;
const updateEventSourceMapping = ({
  id: pipelineId,
  debug = (0, _print.debug)('lambda'),
  step = 'save',
  ...opt
} = {}) => {
  const connector = new _lambda.default({
    pipelineId,
    debug,
    ...opt
  });
  const invoke = uow => {
    if (!uow.updateRequest) return (0, _highland.default)(Promise.resolve(uow));
    const {
      region,
      ...params
    } = uow.updateRequest;
    connector.client.config.region = region || process.env.AWS_REGION;
    const p = () => connector.updateEventSourceMapping(params, uow).then(updateResponse => ({
      ...uow,
      updateResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.flatMap(invoke);
};
exports.updateEventSourceMapping = updateEventSourceMapping;
const deleteEventSourceMapping = ({
  id: pipelineId,
  debug = (0, _print.debug)('lambda'),
  step = 'delete',
  ...opt
} = {}) => {
  const connector = new _lambda.default({
    pipelineId,
    debug,
    ...opt
  });
  const invoke = uow => {
    if (!uow.deleteRequest) return (0, _highland.default)(Promise.resolve(uow));
    const {
      region,
      ...params
    } = uow.deleteRequest;
    connector.client.config.region = region || process.env.AWS_REGION;
    const p = () => connector.deleteEventSourceMapping(params, uow).then(deleteResponse => ({
      ...uow,
      deleteResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.flatMap(invoke);
};
exports.deleteEventSourceMapping = deleteEventSourceMapping;