"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.writeRecords = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _timestream = _interopRequireDefault(require("../connectors/timestream"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
const writeRecords = ({
  // eslint-disable-line import/prefer-default-export
  id: pipelineId,
  debug = (0, _print.debug)('ts'),
  writeRequestField = 'writeRequest',
  writeResponseField = 'writeResponse',
  parallel = Number(process.env.TIMESTREAM_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'save',
  ...opt
} = {}) => {
  const connector = new _timestream.default({
    pipelineId,
    debug,
    ...opt
  });
  const write = uow => {
    // istanbul ignore next
    if (!uow[writeRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.writeRecords(uow[writeRequestField]).then(writeResponse => ({
      ...uow,
      [writeResponseField]: writeResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(write).parallel(parallel);
};
exports.writeRecords = writeRecords;