"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unBatchUow = exports.toGroupUows = exports.toBatchUow = exports.group = exports.compact = exports.batchWithSize = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _isFunction = _interopRequireDefault(require("lodash/isFunction"));
var _claimcheck = require("../sinks/claimcheck");
// used after highland batch step
const toBatchUow = batch => ({
  batch
});

// use with flatMap
exports.toBatchUow = toBatchUow;
const unBatchUow = uow => {
  const {
    batch,
    ...outerUowMinusBatch
  } = uow;
  return batch.map(innerUowFromBatch => ({
    ...innerUowFromBatch,
    ...outerUowMinusBatch
  }));
};
exports.unBatchUow = unBatchUow;
const group = rule => {
  /* istanbul ignore if */
  if (!rule.group) return s => s;
  return s => s.group(uow => uow.event.partitionKey).flatMap(toGroupUows);
};

// use with flatMap after highland group step
exports.group = group;
const toGroupUows = groups => (0, _highland.default)(Object.keys(groups).map(key => ({
  batch: groups[key]
})));
exports.toGroupUows = toGroupUows;
const compact = rule => {
  if (!rule.compact) return s => s;
  return s => (0, _isFunction.default)(rule.compact) && /* istanbul ignore next */rule.compact(s) || s.group(rule.compact.group || (uow => uow.event.partitionKey)).flatMap(groups => (0, _highland.default)(Object.keys(groups).map(key => {
    const batch = groups[key].sort(rule.compact.sort || ((lh, rh) => lh.event.timestamp - rh.event.timestamp));
    const last = batch[batch.length - 1];
    const metrics = last.metrics?.opt.metrics.enabled('compact') ? {
      metrics: last.metrics.gauge('stream.pipeline.compact.count', batch.length)
    } : {};
    return {
      ...last,
      batch,
      ...metrics
    };
  })));
};
exports.compact = compact;
const batchWithSize = ({
  claimCheckBucketName = process.env.CLAIMCHECK_BUCKET_NAME,
  putClaimcheckRequest = 'putClaimcheckRequest',
  ...opt
}) => {
  let batched = [];
  let sizes = [];
  return (err, x, push, next) => {
    /* istanbul ignore if */
    if (err) {
      push(err);
      next();
    } else if (x === _highland.default.nil) {
      if (batched.length > 0) {
        logMetrics(batched, sizes, opt);
        push(null, batched);
      }
      push(null, _highland.default.nil);
    } else {
      if (!x[opt.requestEntryField]) {
        push(null, [x]);
      } else {
        let size = Buffer.byteLength(JSON.stringify(x[opt.requestEntryField]));
        if (size > opt.maxRequestSize) {
          logMetrics([x], [size], opt);
          if (claimCheckBucketName) {
            // setup claim check
            x[putClaimcheckRequest] = (0, _claimcheck.toPutClaimcheckRequest)(JSON.parse(x[opt.requestEntryField].Detail), claimCheckBucketName);
            x[opt.requestEntryField].Detail = JSON.stringify((0, _claimcheck.toClaimcheckEvent)(JSON.parse(x[opt.requestEntryField].Detail), claimCheckBucketName));
            size = Buffer.byteLength(JSON.stringify(x[opt.requestEntryField]));
          } else {
            const error = new Error(`Request size: ${size}, exceeded max: ${opt.maxRequestSize}`);
            error.uow = x;
            push(error);
            next();
            return;
          }
        }
        const totalSize = sizes.reduce((a, c) => a + c, size);
        if (totalSize <= opt.maxRequestSize && batched.length + 1 <= opt.batchSize) {
          batched.push(x);
          sizes.push(size);
        } else {
          logMetrics(batched, sizes, opt);
          push(null, batched);
          batched = [x];
          sizes = [size];
        }
      }
      next();
    }
  };
};
exports.batchWithSize = batchWithSize;
const logMetrics = (batch, sizes, opt) => {
  if (opt.metrics?.enabled('size')) {
    batch[0].metrics?.gauge('publish|stream.pipeline.batchSize.count', batch.length);
    batch[0].metrics?.gauge('publish|stream.pipeline.eventSize.bytes', sizes);
  }
};