"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientLambda = require("@aws-sdk/client-lambda");
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _utilRetry = require("@smithy/util-retry");
var _log = require("./log");
var _print = require("./print");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */
var _default = ({
  functionName,
  endpoint = 'http://localhost:3002',
  timeout = 6000,
  maxAttempt = 8,
  retryDelay = 1000,
  debug = (0, _print.debug)('test')
}) => {
  const lambda = new _clientLambda.LambdaClient({
    endpoint,
    requestHandler: new _nodeHttpHandler.NodeHttpHandler({
      requestTimeout: timeout,
      connectionTimeout: timeout
    }),
    retryStrategy: new _utilRetry.ConfiguredRetryStrategy(maxAttempt,
    // istanbul ignore next
    attempt => 100 + attempt * retryDelay),
    // backoff function for when serverless-offline is slow to start up
    logger: (0, _log.defaultDebugLogger)(debug)
  });
  return event => lambda.send(new _clientLambda.InvokeCommand({
    FunctionName: functionName,
    InvocationType: 'RequestResponse',
    Payload: JSON.stringify(event)
  })).then(resp => ({
    ...resp,
    Payload: JSON.parse(Buffer.from(resp.Payload))
  }));
};
exports.default = _default;